// WARNING: This is an optimized version that fails on hasOwnProperty checks
// and non objects. It's not spec-compliant. It's a perf optimization.
/* eslint strict:0 */
if (!Object.assign) {
  Object.assign = function(target, sources) {
    for (let nextIndex = 1; nextIndex < arguments.length; nextIndex++) {
      let nextSource = arguments[nextIndex];
      if (nextSource == null) {
        continue;
      }

      // We don't currently support accessors nor proxies. Therefore this
      // copy cannot throw. If we ever supported this then we must handle
      // exceptions and side-effects.
      for (let key in nextSource) {
        target[key] = nextSource[key];
      }
    }

    return target;
  };
}

const hasOwnProperty = Object.prototype.hasOwnProperty;

/**
 * Returns an array of the given object's own enumerable entries.
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/entries
 *
 */
if (!Object.entries) {
  Object.entries = function(object) {
    // `null` and `undefined` values are not allowed.
    if (object == null) {
      throw new TypeError('Object.entries called on non-object');
    }

    let entries = [];
    for (let key in object) {
      if (hasOwnProperty.call(object, key)) {
        entries.push([key, object[key]]);
      }
    }
    return entries;
  };
}


/**
 * Returns an array of the given object's own enumerable entries.
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/values
 *
 */
if (!Object.values) {
  Object.values = function(object) {
    // `null` and `undefined` values are not allowed.
    if (object == null) {
      throw new TypeError('Object.values called on non-object');
    }

    let values = [];
    for (let key in object) {
      if (hasOwnProperty.call(object, key)) {
        values.push(object[key]);
      }
    }
    return values;
  };
}

if (!Object.defineProperties) {
  Object.defineProperties = function(object, descriptors) {
    for (var property in descriptors) {
      Object.defineProperty(object, property, descriptors[property]);
    }
    return object;
  };
}

// https://gist.github.com/WebReflection/5593554
if (!Object.setPrototypeOf) {
  Object.setPrototypeOf = (function(Object, magic) {
    var set;
    function setPrototypeOf(O, proto) {
      set.call(O, proto);
      return O;
    }
    try {
      // this works already in Firefox and Safari
      set = Object.getOwnPropertyDescriptor(Object.prototype, magic).set;
      set.call({}, null);
    } catch (e) {
      if (
        // IE < 11 cannot be shimmed
        Object.prototype !== {}[magic] ||
        // neither can any browser that actually
        // implemented __proto__ correctly
        // (all but old V8 will return here)
        {__proto__: null}.__proto__ === void 0
        // this case means null objects cannot be passed
        // through setPrototypeOf in a reliable way
        // which means here a **Sham** is needed instead
      ) {
        return;
      }
      // nodejs 0.8 and 0.10 are (buggy and..) fine here
      // probably Chrome or some old Mobile stock browser
      set = function(proto) {
        this[magic] = proto;
      };
      // please note that this will **not** work
      // in those browsers that do not inherit
      // __proto__ by mistake from Object.prototype
      // in these cases we should probably throw an error
      // or at least be informed about the issue
      setPrototypeOf.polyfill = setPrototypeOf(
        setPrototypeOf({}, null),
        Object.prototype
      ) instanceof Object;
      // setPrototypeOf.polyfill === true means it works as meant
      // setPrototypeOf.polyfill === false means it's not 100% reliable
      // setPrototypeOf.polyfill === undefined
      // or
      // setPrototypeOf.polyfill ==  null means it's not a polyfill
      // which means it works as expected
      // we can even delete Object.prototype.__proto__;
    }
    return setPrototypeOf;
  }(Object, '__proto__'));
}

if (!Object.is) {
  Object.is = function(x, y) {
    if (x === y) {
      // 0 === -0, but they are not identical
      return x !== 0 || 1 / x === 1 / y;
    }

    // NaN !== NaN, but they are identical.
    // NaNs are the only non-reflexive value, i.e., if x !== x,
    // then x is a NaN.
    // isNaN is broken: it converts its argument to number, so
    // isNaN("foo") => true
    return x !== x && y !== y;
  };
}
